# Esercizio "Magazzino automatico" (PLI)

# DATI
param nR;		    # Numero di righe
param nC;		    # Numero di colonne
param o;		    # Posizione dell'origine
param h;		    # Dimensione orizzontale [m]
param v;		    # Dimensione verticale [m]
param nP;		    # Numero dei punti di prelievo
param nD;		    # Numero dei punti di consegna
set P := 1..nP;	    # Insieme di pickup
set D := 1..nD;	    # Insieme di delivery
param sitoP {P};	# Siti di prelievo
param sitoD {D};	# Siti di consegna

# Coordinate (riga,colonna) dei punti di prelievo e di consegna
param rowP {p in P} := (sitoP[p]-1 div nC) +1;
param rowD {d in D} := (sitoD[d]-1 div nC) +1;
param colP {p in P} := (sitoP[p]-1 mod nC) +1;
param colD {d in D} := (sitoD[d]-1 mod nC) +1;
# Coordinate del punto di origine
param row0 := (o-1 div nC) +1;
param col0 := (o-1 mod nC) +1;
# Distanze tra i punti
param doP {p in P} := sqrt((h*(col0-colP[p]))^2+(v*(row0-rowP[p]))^2);
param doD {d in D} := sqrt((h*(col0-colD[d]))^2+(v*(row0-rowD[d]))^2);
param dPo {p in P} := sqrt((h*(col0-colP[p]))^2+(v*(row0-rowP[p]))^2);
param dDo {d in D} := sqrt((h*(col0-colD[d]))^2+(v*(row0-rowD[d]))^2);
param dPP {p1 in P,p2 in P} := sqrt((h*(colP[p1]-colP[p2]))^2+(v*(rowP[p1]-rowP[p2]))^2);
param dDD {d1 in D,d2 in D} := sqrt((h*(colD[d1]-colD[d2]))^2+(v*(rowD[d1]-rowD[d2]))^2);
param dDP {d in D,p in P}   := sqrt((h*(colP[p] -colD[d] ))^2+(v*(rowP[p] -rowD[d] ))^2);

# VARIABILI
var xoP {P} binary;
var xoD {D} binary;
var xPo {P} binary;
var xDo {D} binary;
var xPP {p1 in P, p2 in P: p1 <> p2} binary;
var xDD {d1 in D, d2 in D: d1 <> d2} binary;
var xDP {D,P} binary;

# VINCOLI 
# Ogni ordine deve essere soddisfatto: vincoli di grado
subject to IndegreeP {p in P}:
  sum {p2 in P: p2<>p} xPP[p2,p] + sum {d in D} xDP[d,p] + xoP[p] = 1;
subject to OutdegreeP {p in P}:
  sum {p2 in P: p2<>p} xPP[p,p2] + xPo[p] = 1;
subject to IndegreeD {d in D}:
  sum {d2 in D: d2<>d} xDD[d2,d] + xoD[d] = 1;
subject to OutdegreeD {d in D}:
  sum {d2 in D: d2<>d} xDD[d,d2] + sum {p in P} xDP[d,p] + xDo[d] = 1;

# No terne P
subject to No3P {p1 in P, p2 in P, p3 in P: p1<p2 and p2<p3}:
  xPP[p1,p2] + xPP[p2,p3] + xPP[p1,p3] +
  xPP[p2,p1] + xPP[p3,p2] + xPP[p3,p1] <= 1;
# No terne D
subject to No3D {d1 in D, d2 in D, d3 in D: d1<d2 and d2<d3}:
  xDD[d1,d2] + xDD[d2,d3] + xDD[d1,d3] +
  xDD[d2,d1] + xDD[d3,d2] + xDD[d3,d1] <= 1;

# OBIETTIVO
# Minimizzare la distanza totale [m]
minimize z: sum {p in P} (xoP[p] * doP[p] + xPo[p] * doP[p]) + 
            sum {d in D} (xoD[d] * doD[d] + xDo[d] * doD[d]) + 
            sum {p1 in P, p2 in P: p1<>p2} (xPP[p1,p2] * dPP[p1,p2]) + 
            sum {d1 in D, d2 in D: d1<>d2} (xDD[d1,d2] * dDD[d1,d2]) + 
            sum {p in P, d in D} (xDP[d,p] * dDP[d,p]);

########################
data;

param nR :=  5;
param nC := 13;
param o  := 33;
param h  :=  1;
param v  :=  0.6;

param nD := 9;
param sitoD:=
1	24
2	39
3	12
4	60
5	48
6	49
7	42
8	19
9	5;

param nP := 7;
param sitoP :=
1	38
2	26
3	11
4	9
5	63
6	18
7	55;

end;
